//+------------------------------------------------------------------+
//|                                                   Tradecheck.mqh |
//|                        Copyright 2017, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property strict

bool CheckOpenOrder(string symb, int tf, double lots, ENUM_ORDER_TYPE oper, double SL, double TP, double price, bool CheckNewBar)
{
   //if(ClosedOrdersPerBar(oper)>0) return(false);


// We will use the static Old_Time variable to serve the bar time.
// At each OnTick execution we will check the current bar time with the saved one.
// If the bar time isn't equal to the saved time, it indicates that we have a new tick.   
   if(CheckNewBar){
      static datetime Old_Time;
      datetime New_Time[1];
      bool IsNewBar=false;
      // copying the last bar time to the element New_Time[0]
      int copied=CopyTime(symb,(ENUM_TIMEFRAMES)tf,0,1,New_Time);
      if(copied>0) // ok, the data has been copied successfully
        {
         if(Old_Time!=New_Time[0]) // if old time isn't equal to new bar time
           {
            IsNewBar=true;   // if it isn't a first call, the new bar has appeared
            //if(MQLInfoInteger(MQL_DEBUG)) Print("We have new bar here ",New_Time[0]," old time was ",Old_Time);
            Old_Time=New_Time[0];            // saving bar time
           }
        }
      else
        {
         Alert("Error in copying historical time data for ",symb + " " + IntegerToString(tf), ", error =",GetLastError());
         ResetLastError();
         return(false);
        }
   
      //--- EA should only check for new trade if we have a new bar
      if(IsNewBar==false) return(false);
   }
     
   //--- Do we have enough bars to work with
   int Mybars=iBars(symb,(ENUM_TIMEFRAMES)tf);
   if(Mybars<10)
     {
      Alert("We have less than 10 bars on ",symb, ", EA will not take any trade!!");
      return(false);
     }
     
   string text;
   if(!CheckMoneyForTrade(symb,lots,oper)) return(false);
   if(!CheckVolumeValue(symb,lots,text)){
      Print(text);
      return(false);
   }
   if(!IsNewOrderAllowed()) return(false);
   if(!CheckStopLoss_Takeprofit(symb, oper, price, SL, TP)) return(false);
   
   return(true);   
}

bool CheckMoneyForTrade(string symb,double lots,ENUM_ORDER_TYPE type)
 {
//--- Getting the opening price
   MqlTick mqltick;
   SymbolInfoTick(symb,mqltick);
   double price=mqltick.ask;
   if(type==ORDER_TYPE_SELL)
      price=mqltick.bid;
//--- values of the required and free margin
   double margin,free_margin=AccountInfoDouble(ACCOUNT_MARGIN_FREE);
   //--- call of the checking function
   if(!OrderCalcMargin(type,symb,lots,price,margin))
     {
      //--- something went wrong, report and return false
      Print("Error in ",__FUNCTION__," code=",GetLastError());
      return(false);
     }
   //--- if there are insufficient funds to perform the operation
   if(margin>free_margin)
     {
      //--- report the error and return false
      Print("Not enough money for ",EnumToString(type)," ",lots," ",symb," Error code=",GetLastError());
      return(false);
     }
//--- checking successful
   return(true);
 }

//+------------------------------------------------------------------+
//| Check the correctness of the order volume                        |
//+------------------------------------------------------------------+
bool CheckVolumeValue(string symb, double volume,string &description)
  {
//--- minimal allowed volume for trade operations
   double min_volume=SymbolInfoDouble(symb,SYMBOL_VOLUME_MIN);
   if(volume<min_volume)
     {
      description=StringFormat("Volume %.3f is less than the minimal allowed SYMBOL_VOLUME_MIN=%.2f",volume,min_volume);
      return(false);
     }

//--- maximal allowed volume of trade operations
   double max_volume=SymbolInfoDouble(symb,SYMBOL_VOLUME_MAX);
   if(volume>max_volume)
     {
      description=StringFormat("Volume %.3f is greater than the maximal allowed SYMBOL_VOLUME_MAX=%.2f",volume,max_volume);
      return(false);
     }

//--- get minimal step of volume changing
   double volume_step=SymbolInfoDouble(symb,SYMBOL_VOLUME_STEP);

   int ratio=(int)MathRound(volume/volume_step);
   if(MathAbs(ratio*volume_step-volume)>0.0000001)
     {
      description=StringFormat("Volume %.3f is not a multiple of the minimal step SYMBOL_VOLUME_STEP=%.2f, the closest correct volume is %.2f",volume,volume_step,ratio*volume_step);
      return(false);
     }
   description="Correct volume value";
   return(true);
  }
  
  
//+------------------------------------------------------------------+
//| Check if another order can be placed                             |
//+------------------------------------------------------------------+
bool IsNewOrderAllowed()
  {
//--- get the number of pending orders allowed on the account
   int max_allowed_orders=(int)AccountInfoInteger(ACCOUNT_LIMIT_ORDERS);

//--- if there is no limitation, return true; you can send an order
   if(max_allowed_orders==0) return(true);

//--- if we passed to this line, then there is a limitation; find out how many orders are already placed
   int orders=OrdersTotal();

//--- return the result of comparing
   //return(orders<max_allowed_orders);
   
   if(orders<max_allowed_orders) return(true);
   Print("You have reached the limit of opened orders. Error code=",GetLastError());
   return(false);
  }

/*
//--- calculate allowed volume for another pending order
   double new_order_volume=NewOrderAllowedVolume(Symbol());
   string com="";
   StringAdd(com,"Maximal allowed volume of the new order by "+Symbol()+", lots: "+DoubleToString(new_order_volume,2));
*/
//+------------------------------------------------------------------+
//| Return the maximum allowed volume for an order on the symbol     |
//+------------------------------------------------------------------+
double NewOrderAllowedVolume(string symb)
  {
   double allowed_volume=0;
//--- get the limitation on the maximal volume of an order
   double symbol_max_volume=SymbolInfoDouble(symb,SYMBOL_VOLUME_MAX);
//--- get the limitation on the volume by a symbol
   double max_volume=SymbolInfoDouble(symb,SYMBOL_VOLUME_LIMIT);

//--- get the volume of the open position by a symbol
   double opened_volume=PositionVol(symb);
   if(opened_volume>=0)
     {
      //--- if we have exhausted the volume
      if(max_volume-opened_volume<=0)
         return(0);

      //--- volume of the open position doesn't exceed max_volume
      double orders_volume_on_symbol=PendingsVolume(symb);
      allowed_volume=max_volume-opened_volume-orders_volume_on_symbol;
      if(allowed_volume>symbol_max_volume) allowed_volume=symbol_max_volume;
     }
   return(allowed_volume);
  }

//+------------------------------------------------------------------+
//| Return the size of position on the specified symbol              |
//+------------------------------------------------------------------+
double PositionVol(string symb)
 {
//--- try to select position by a symbol
   bool selected=PositionSelect(symb);
//--- there is a position
   if(selected)
      //--- return volume of the position
      return(PositionGetDouble(POSITION_VOLUME));
   else
     {
      //--- report a failure to select position
      Print(__FUNCTION__," Failed to perform PositionSelect() for symbol ",
            symb," Error ",GetLastError());
      return(-1);
     }
 }

//+------------------------------------------------------------------+
//|  returns the volume of current pending order by a symbol         |
//+------------------------------------------------------------------+
double PendingsVolume(string symb)
 {
   double volume_on_symbol=0;
   ulong ticket;
//---  get the number of all currently placed orders by all symbols
   int all_orders=OrdersTotal();

//--- get over all orders in the loop
   for(int i=0;i<all_orders;i++)
     {
      //--- get the ticket of an order by its position in the list
      if((ticket=OrderGetTicket(i))>0)
        {
         //--- if our symbol is specified in the order, add the volume of this order
         if(symb==OrderGetString(ORDER_SYMBOL))
            volume_on_symbol+=OrderGetDouble(ORDER_VOLUME_INITIAL);
        }
     }
//--- return the total volume of currently placed pending orders for a specified symbol
   return(volume_on_symbol);
 }
  

bool CheckStopLoss_Takeprofit(string symb,ENUM_ORDER_TYPE type,double price,double SL,double TP)
 {
    string oper=(type==ORDER_TYPE_BUY || type==ORDER_TYPE_BUY_STOP || type==ORDER_TYPE_BUY_LIMIT)? "Buy":"Sell";

//--- get the SYMBOL_TRADE_STOPS_LEVEL level
   int stops_level=(int)SymbolInfoInteger(symb,SYMBOL_TRADE_STOPS_LEVEL);
   double ask = SymbolInfoDouble(symb,SYMBOL_ASK);
   double bid = SymbolInfoDouble(symb,SYMBOL_BID);
   double point = SymbolInfoDouble(symb,SYMBOL_POINT);
   if(stops_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_STOPS_LEVEL=%d: StopLoss and TakeProfit must"+
                  " not be nearer than %d points from the closing price",stops_level,stops_level);
     }
//---
   bool SL_check=false,TP_check=false;
//--- check the order type
   switch(type)
     {
      //--- Buy operation
      case  ORDER_TYPE_BUY:
        {
         //--- check the StopLoss
         SL_check=(bid-SL>stops_level*point);
         if(SL==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("For %s order %s   StopLoss=%.5f must be less than %.5f"+
                        " (Bid=%.5f - SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,SL,bid-stops_level*point,bid,stops_level);
         //--- check the TakeProfit
         TP_check=(TP-bid>stops_level*point);
         if(TP==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("For %s order %s   TakeProfit=%.5f must be greater than %.5f"+
                        " (Bid=%.5f + SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,TP,bid+stops_level*point,bid,stops_level);
         //--- return the result of checking
         return(SL_check&&TP_check);
        }
      //--- Sell operation
      case  ORDER_TYPE_SELL:
        {
         //--- check the StopLoss
         SL_check=(SL-ask>stops_level*point);
         if(SL==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("For %s order %s   StopLoss=%.5f must be greater than %.5f"+
                        " (Ask=%.5f + SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,SL,ask+stops_level*point,ask,stops_level);
         //--- check the TakeProfit
         TP_check=(ask-TP>stops_level*point);
         if(TP==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("For %s order %s   TakeProfit=%.5f must be less than %.5f"+
                        " (Ask=%.5f - SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,TP,ask-stops_level*point,ask,stops_level);
         //--- return the result of checking
         return(TP_check&&SL_check);
        }
      break;
      //--- BuyLimit pending order
      case  ORDER_TYPE_BUY_LIMIT:
        {
         //--- check the StopLoss
         SL_check=((price-SL)>stops_level*point);
         if(SL==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("For %s order %s   StopLoss=%.5f must be less than %.5f"+
                        " (Open-StopLoss=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,SL,price-stops_level*point,(int)((price-SL)/point),stops_level);
         //--- check the TakeProfit
         TP_check=((TP-price)>stops_level*point);
         if(TP==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("For %s order %s   TakeProfit=%.5f must be greater than %.5f"+
                        " (TakeProfit-Open=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,TP,price+stops_level*point,(int)((TP-price)/point),stops_level);
         //--- return the result of checking
         return(SL_check&&TP_check);
        }
      //--- SellLimit pending order
      case  ORDER_TYPE_SELL_LIMIT:
        {
         //--- check the StopLoss
         SL_check=((SL-price)>stops_level*point);
         if(SL==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("For %s order %s   StopLoss=%.5f must be greater than %.5f"+
                        " (StopLoss-Open=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,SL,price+stops_level*point,(int)((SL-price)/point),stops_level);
         //--- check the TakeProfit
         TP_check=((price-TP)>stops_level*point);
         if(TP==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("For %s order %s   TakeProfit=%.5f must be less than %.5f"+
                        " (Open-TakeProfit=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,TP,price-stops_level*point,(int)((price-TP)/point),stops_level);
         //--- return the result of checking
         return(TP_check&&SL_check);
        }
      break;
      //--- BuyStop pending order
      case  ORDER_TYPE_BUY_STOP:
        {
         //--- check the StopLoss
         SL_check=((price-SL)>stops_level*point);
         if(SL==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("For %s order %s   StopLoss=%.5f must be less than %.5f"+
                        " (Open-StopLoss=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,SL,price-stops_level*point,(int)((price-SL)/point),stops_level);
         //--- check the TakeProfit
         TP_check=((TP-price)>stops_level*point);
         if(TP==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("For %s order %s   TakeProfit=%.5f must be greater than %.5f"+
                        " (TakeProfit-Open=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,TP,price-stops_level*point,(int)((TP-price)/point),stops_level);
         //--- return the result of checking
         return(SL_check&&TP_check);
        }
      //--- SellStop pending order
      case  ORDER_TYPE_SELL_STOP:
        {
         //--- check the StopLoss
         SL_check=((SL-price)>stops_level*point);
         if(SL==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("For %s order %s   StopLoss=%.5f must be greater than %.5f"+
                        " (StopLoss-Open=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,SL,price+stops_level*point,(int)((SL-price)/point),stops_level);
         //--- check the TakeProfit
         TP_check=((price-TP)>stops_level*point);
         if(TP==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("For %s order %s   TakeProfit=%.5f must be less than %.5f"+
                        " (Open-TakeProfit=%d points ==> SYMBOL_TRADE_STOPS_LEVEL=%d points)",
                        symb,oper,TP,price-stops_level*point,(int)((price-TP)/point),stops_level);
         //--- return the result of checking
         return(TP_check&&SL_check);
        }
      break;
     }

//---
   return false;
 }
  
  
bool CheckOrderForFREEZE_LEVEL(ulong ticket)
 {
//--- get the SYMBOL_TRADE_FREEZE_LEVEL level
   //--- select order for working
   if(!PositionSelectByTicket(ticket)) return(false); //--- failed to select order
   
   int freeze_level=(int)SymbolInfoInteger(PositionGetString(POSITION_SYMBOL),SYMBOL_TRADE_FREEZE_LEVEL);   
   if(freeze_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_FREEZE_LEVEL=%d: Cannot modify order"+
                  "  nearer than %d points from the activation price",freeze_level,freeze_level);
     }

//--- get the order data
   double price=PositionGetDouble(POSITION_PRICE_OPEN);
   double sl=PositionGetDouble(POSITION_SL);
   double tp=PositionGetDouble(POSITION_TP);
   string symb=PositionGetString(POSITION_SYMBOL);
   ENUM_ORDER_TYPE type=(ENUM_ORDER_TYPE)PositionGetInteger(POSITION_TYPE);
   double ask = SymbolInfoDouble(symb,SYMBOL_ASK);
   double bid = SymbolInfoDouble(symb,SYMBOL_BID);
   double point = SymbolInfoDouble(symb,SYMBOL_POINT);
   
//--- result of checking 
   bool check=false;
//--- check the order type
   switch(type)
     {
      //--- BuyLimit pending order
      case  ORDER_TYPE_BUY_LIMIT:
        {
         //--- check the distance from the opening price to the activation price
         check=((ask-price)>freeze_level*point);
         if(!check)
            PrintFormat("Order OP_BUYLIMIT #%d cannot be modified: Ask-Open=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((ask-price)/point),freeze_level);
         return(check);
        }
      //--- BuyLimit pending order
      case  ORDER_TYPE_SELL_LIMIT:
        {
         //--- check the distance from the opening price to the activation price
         check=((price-bid)>freeze_level*point);
         if(!check)
            PrintFormat("Order OP_SELLLIMIT #%d cannot be modified: Open-Bid=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((price-bid)/point),freeze_level);
         return(check);
        }
      break;
      //--- BuyStop pending order
      case  ORDER_TYPE_BUY_STOP:
        {
         //--- check the distance from the opening price to the activation price
         check=((price-ask)>freeze_level*point);
         if(!check)
            PrintFormat("Order OP_BUYSTOP #%d cannot be modified: Ask-Open=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((price-ask)/point),freeze_level);
         return(check);
        }
      //--- SellStop pending order
      case  ORDER_TYPE_SELL_STOP:
        {
         //--- check the distance from the opening price to the activation price
         check=((bid-price)>freeze_level*point);
         if(!check)
            PrintFormat("Order OP_SELLSTOP #%d cannot be modified: Bid-Open=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((bid-price)/point),freeze_level);
         return(check);
        }
      break;
      //--- checking opened Buy order
      case  ORDER_TYPE_BUY:
        {
         //--- check TakeProfit distance to the activation price
         bool TP_check=(tp-bid>freeze_level*point);
         if(tp==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("Order OP_BUY %d cannot be modified: TakeProfit-Bid=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((tp-bid)/point),freeze_level);
         //--- check TakeProfit distance to the activation price
         bool SL_check=(bid-sl>freeze_level*point);
         if(sl==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("Order OP_BUY %d cannot be modified: TakeProfit-Bid=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((bid-sl)/point),freeze_level);
         return(SL_check&&TP_check);
        }
      break;
      //--- checking opened Sell order
      case  ORDER_TYPE_SELL:
        {
         //--- check TakeProfit distance to the activation price
         bool TP_check=(ask-tp>freeze_level*point);
         if(tp==0.0) TP_check=true;
         if(!TP_check)
            PrintFormat("Order OP_SELL %d cannot be modified: Ask-TakeProfit=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((ask-tp)/point),freeze_level);
         //--- check TakeProfit distance to the activation price
         bool SL_check=(sl-ask>freeze_level*point);
         if(sl==0.0) SL_check=true;
         if(!SL_check)
            PrintFormat("Order OP_BUY %d cannot be modified: TakeProfit-Bid=%d points < SYMBOL_TRADE_FREEZE_LEVEL=%d points",
                        ticket,(int)((sl-ask)/point),freeze_level);
         return(SL_check&&TP_check);
        }
      break;
     }
//--- order did not pass the check
   return (false);
 }

/*
bool OrderModifyCheck(ulong ticket,double price,double sl,double tp)
  {
//--- select order by ticket
   if(orderinfo.Select(ticket))
     {
      //--- point size and name of the symbol, for which a pending order was placed
      string symbol=orderinfo.Symbol();
      double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
      int digits=(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
      //--- check if there are changes in the Open price
      bool PriceOpenChanged=(MathAbs(orderinfo.PriceOpen()-price)>point);
      //--- check if there are changes in the StopLoss level
      bool StopLossChanged=(MathAbs(orderinfo.StopLoss()-sl)>point);
      //--- check if there are changes in the Takeprofit level
      bool TakeProfitChanged=(MathAbs(orderinfo.TakeProfit()-tp)>point);
      //--- if there are any changes in levels
      if(PriceOpenChanged || StopLossChanged || TakeProfitChanged)
         return(true);  // order can be modified      
      //--- there are no changes in the Open, StopLoss and Takeprofit levels
      else
      //--- notify about the error
         PrintFormat("Order #%d already has levels of Open=%.5f SL=%.5f TP=%.5f",
                     ticket,orderinfo.PriceOpen(),orderinfo.StopLoss(),orderinfo.TakeProfit());
     }
//--- came to the end, no changes for the order
   return(false);       // no point in modifying 
 }
 */

//+------------------------------------------------------------------+
//| Checking the new values of levels before order modification      |
//+------------------------------------------------------------------+
bool OrderModifyCheck(ulong ticket,double price,double sl,double tp)
 {
//--- select order by ticket
   if(PositionSelectByTicket(ticket))
     {
      //--- point size and name of the symbol, for which a pending order was placed
      string symb=PositionGetString(POSITION_SYMBOL);
      double point=SymbolInfoDouble(symb,SYMBOL_POINT);
      //--- check if there are changes in the Open price
      bool PriceOpenChanged=false;
      ENUM_ORDER_TYPE type=(ENUM_ORDER_TYPE)PositionGetInteger(POSITION_TYPE);
      
      if(!(type==ORDER_TYPE_BUY || type==ORDER_TYPE_SELL))
        {
         PriceOpenChanged=(MathAbs(PositionGetDouble(POSITION_PRICE_OPEN)-price)>point);
        }
      //--- check if there are changes in the StopLoss level
      bool StopLossChanged=(MathAbs(PositionGetDouble(POSITION_SL)-sl)>point);
      //--- check if there are changes in the Takeprofit level
      bool TakeProfitChanged=(MathAbs(PositionGetDouble(POSITION_TP)-tp)>point);
      //--- if there are any changes in levels
      if(PriceOpenChanged || StopLossChanged || TakeProfitChanged) return(true);  // order can be modified  
      //if(StopLossChanged || TakeProfitChanged) return(true);  // order can be modified   
      //--- there are no changes in the Open, StopLoss and Takeprofit levels
      else
      //--- notify about the error
         PrintFormat("Order #%d already has levels of open/SL/TP=%.5f SL=%.5f TP=%.5f",
                     ticket,PositionGetDouble(POSITION_PRICE_OPEN),PositionGetDouble(POSITION_SL),PositionGetDouble(POSITION_TP));
     }
//--- came to the end, no changes for the order
   return(false);       // no point in modifying 
 }
  
  
/////////////////////////////////////////////////////////////////////////////////
// int _IsTradeAllowed( int MaxWaiting_sec = 30 )
//
// the function checks the trade context status. Return codes:
//  1 - trade context is free, trade allowed
//  0 - trade context was busy, but became free. Trade is allowed only after 
//      the market info has been refreshed.
// -1 - trade context is busy, waiting interrupted by the user (expert was removed from 
//      the chart, terminal was shut down, the chart period and/or symbol was changed, etc.)
// -2 - trade context is busy, the waiting limit is reached (MaxWaiting_sec). 
//      Possibly, the expert is not allowed to trade (checkbox "Allow live trading" 
//      in the expert settings).
//
// MaxWaiting_sec - time (in seconds) within which the function will wait 
// until the trade context is free (if it is busy). By default,30.
/////////////////////////////////////////////////////////////////////////////////
int _IsTradeAllowed(uint MaxWaiting_sec = 30)
 {
    // check whether the trade context is free
    if(MQLInfoInteger(MQL_TRADE_ALLOWED)==0)
      {
        uint StartWaitingTime = GetTickCount();
        Print("Trade context is busy! Wait until it is free...");
        // infinite loop
        while(true)
          {
            // if the expert was terminated by the user, stop operation
            if(IsStopped()) 
              { 
                Print("The expert was terminated by the user!"); 
                return(-1); 
              }
            // if the waiting time exceeds the time specified in the 
            // MaxWaiting_sec variable, stop operation, as well
            if(GetTickCount() - StartWaitingTime > MaxWaiting_sec * 1000)
              {
                Print("The waiting limit exceeded (" + IntegerToString(MaxWaiting_sec) + " seconds)!");
                return(-2);
              }
            // if the trade context has become free,
            if(MQLInfoInteger(MQL_TRADE_ALLOWED)==1)
              {
                Print("Trade context has become free!");
                return(0);
              }
              // if no loop breaking condition has been met, "wait" for 0.1 
              // second and then restart checking
              Sleep(100);
          }
      }
    else
      {
        //Print("Trade context is free!");
        return(1);
      }
 }